@@ -31,8 +31,6 @@
 # with Ubiquity; if not, write to the Free Software Foundation, Inc., 51
 # Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-from __future__ import print_function
-
 import atexit
 import configparser
 from functools import reduce
@@ -51,6 +49,12 @@ DBusGMainLoop(set_as_default=True)
 
 # in query mode we won't be in X, but import needs to pass
 if 'DISPLAY' in os.environ:
+    gi.require_version('Atk', '1.0')
+    gi.require_version('Gdk', '3.0')
+    gi.require_version('Gtk', '3.0')
+    gi.require_version('Gio', '2.0')
+    gi.require_version('GLib', '2.0')
+    gi.require_version('GObject', '2.0')
     from gi.repository import Gtk, Gdk, GObject, GLib, Atk, Gio
     from ubiquity import gtkwidgets
 
@@ -402,13 +406,19 @@ class Wizard(BaseFrontend):
             self.live_installer.connect(
                 'key-press-event', self.a11y_profile_keys)
 
+        # Wait for sound.target to become ready
+        self.soundWatcher = misc.SystemdUnitWatcher('sound.target',
+                                                    self.play_system_ready)
+
+        self.save_oem_metapackages_list()
+
+    def play_system_ready(self):
+        # play the system ready sound
         if osextras.find_on_path('canberra-gtk-play'):
             subprocess.Popen(
                 ['canberra-gtk-play', '--id=system-ready'],
                 preexec_fn=misc.drop_all_privileges)
 
-        self.save_oem_metapackages_list()
-
     def save_oem_metapackages_list(self, wait_finished=False):
         ''' If we can, update the apt indexes. Then run 'ubuntu-drivers
         list-oem' to find any OEM metapackages for this system. If we've
@@ -570,6 +580,11 @@ class Wizard(BaseFrontend):
             GLib.source_remove(self.timeout_id)
         self.timeout_id = GLib.timeout_add(300, self.check_returncode)
 
+    def set_connectivity_state(self, state):
+        for p in self.pages:
+            if hasattr(p.ui, 'plugin_set_connectivity_state'):
+                p.ui.plugin_set_connectivity_state(state)
+
     def set_online_state(self, state):
         for p in self.pages:
             if hasattr(p.ui, 'plugin_set_online_state'):
@@ -954,9 +969,7 @@ class Wizard(BaseFrontend):
             request = decision.get_request()
             uri = request.get_uri()
             decision.ignore()
-            subprocess.Popen(['sensible-browser', uri],
-                             close_fds=True,
-                             preexec_fn=misc.drop_all_privileges)
+            misc.launch_uri(uri)
             return True
         return False
 
@@ -1030,6 +1043,7 @@ class Wizard(BaseFrontend):
         # TODO lazy load
         import gi
         gi.require_version("Vte", "2.91")
+        gi.require_version("Pango", "1.0")
         from gi.repository import Vte, Pango
         misc.drop_privileges_save()
         self.vte = Vte.Terminal()
@@ -1117,6 +1131,7 @@ class Wizard(BaseFrontend):
         self.allow_go_backward(False)
 
         misc.add_connection_watch(self.network_change)
+        misc.add_connection_watch(self.set_connectivity_state, global_only=False)
 
     def set_window_hints(self, widget):
         if (self.oem_user_config or
